"""
This module is used to run all the doctests for all the modules in a given directory.

- :py:func:`~main.launch_tests` - Main function to launch tests for all modules in the given directory.
- :py:func:`~utils.test_module_with_progress` - Test a module with testmod and measure the time taken with progress printing.

.. image:: https://raw.githubusercontent.com/Stoupy51/stouputils/refs/heads/main/assets/all_doctests_module.gif
  :alt: stouputils all_doctests examples
"""

# Imports
from .main import *
from .utils import *

